@@ -24,7 +24,9 @@
 #' height when calling `complecs` to generate a COMPLECS overview.
 #' @param width,height If not `NULL`, a way to override the width and
 #' height when calling `print` to print a COMPLECS overview.
+#' @param returnDotOnly Whether to only return the produced DOT code.
 #' @param returnSvgOnly Whether to only return the SVG in a character vector.
+#' @param returnGraphOnly Whether to only return the produced graph.
 #' @param maxLabelLength The number of characters where to wrap the labels.
 #' @param regExReplacements A list of pairs of regular
 #'   expressions that will be applied to the specifications
@@ -76,7 +78,9 @@ complecs <- function(input,
                      outputFile = NULL,
                      outputWidth=1600,
                      outputHeight=NULL,
+                     returnDotOnly = FALSE,
                      returnSvgOnly = FALSE,
+                     returnGraphOnly = TRUE,
                      maxLabelLength=20,
                      regExReplacements = opts$get("diagrammerSanitization"),
                      silent = opts$get("silent")) {
@@ -211,10 +215,12 @@ complecs <- function(input,
   entityTypes <- entityTypes[(nchar(entityTypes$entity_type_id) > 0), ];
   connectionTypes <- connectionTypes[(nchar(connectionTypes$connection_type_id) > 0), ];
 
-  ### Sanitize
+  ### Sanitize attributes and identifiers
+
   entities <- sanitizeForDiagrammer(
     entities,
-    columns = entityCols['entity_label'],
+    columns = names(entities),
+    #columns = entityCols['entity_label'],
     regExReplacements = regExReplacements);
 
   entityTypes <- sanitizeForDiagrammer(
@@ -227,9 +233,21 @@ complecs <- function(input,
     columns = connectionTypesCols['connection_type_label'],
     regExReplacements = regExReplacements);
 
+  entities <- sanitizeForDiagrammer(
+    entities,
+    columns = entityCols[c('entity_id', 'parent_id')],
+    regExReplacements = list(c("[^a-zA-Z0-9_]", "")));
+
+  connections <- sanitizeForDiagrammer(
+    connections,
+    columns = connectionsCols[c('from_entity_id', 'to_entity_id')],
+    regExReplacements = list(c("[^a-zA-Z0-9_]", "")));
+
   msg("Sanitized the input!\n", silent=silent);
 
-  msg("Looking for nonexistent entities... ", silent=silent);
+  ### Checking integrity
+
+  msg("Looking for nonexistent entities...\n", silent=silent);
 
   fromConnections_nonexistentEntities <-
     connections$from_entity_id[
@@ -252,19 +270,31 @@ complecs <- function(input,
           connectionTypes$connection_type_id)];
 
   if (length(fromConnections_nonexistentEntities) > 0) {
-    warning("In the connections sheet, connections are specified ",
-            "from the following entities that are absent from ",
-            "the entities sheet: ",
-            vecTxtQ(unique(fromConnections_nonexistentEntities)),
-            ".");
+    message <-
+      paste0("In the connections sheet, connections are specified ",
+             "from the following entities that are absent from ",
+             "the entities sheet: ",
+             vecTxtQ(unique(fromConnections_nonexistentEntities)),
+             ".\n");
+    if (!silent) {
+      msg(message, silent=silent);
+    } else {
+      warning(message);
+    }
   }
 
   if (length(toConnections_nonexistentEntities) > 0) {
-    warning("In the connections sheet, connections are specified ",
-            "from the following entities that are absent from ",
-            "the entities sheet: ",
-            vecTxtQ(unique(toConnections_nonexistentEntities)),
-            ".");
+    message <-
+      paste0("In the connections sheet, connections are specified ",
+             "to the following entities that are absent from ",
+             "the entities sheet: ",
+             vecTxtQ(unique(toConnections_nonexistentEntities)),
+             ".\n");
+    if (!silent) {
+      msg(message, silent=silent);
+    } else {
+      warning(message);
+    }
   }
 
   if ((length(fromConnections_nonexistentEntities) == 0) &
@@ -464,6 +494,14 @@ complecs <- function(input,
 
   ### Connections
 
+  ### First fix NAs
+  mergedConnections[, connectionTypesCols['connection_label']] <-
+    ifelse(
+      is.na(mergedConnections[, connectionTypesCols['connection_label']]),
+      "",
+      mergedConnections[, connectionTypesCols['connection_label']]
+    );
+
   for (currentRow in 1:nrow(mergedConnections)) {
 
     dot_code <-
@@ -474,7 +512,7 @@ complecs <- function(input,
         " -> ",
         mergedConnections[currentRow, connectionsCols['to_entity_id']],
         " [label=\"",
-        mergedConnections[currentRow, connectionsCols['connection_label']],
+        mergedConnections[currentRow, connectionTypesCols['connection_label']],
         "\", ",
         paste0(connectionAttributes, "=\"",
                mergedConnections[currentRow, connectionAttributes],
@@ -547,6 +585,10 @@ complecs <- function(input,
   # ### From DiagrammeR::export_graph
   # dot_code <- DiagrammeR::generate_dot(graph);
 
+  if (returnDotOnly) {
+    return(dot_code);
+  }
+
   graph <- DiagrammeR::grViz(dot_code,
                              engine = layout);
   graphSvg <-
@@ -583,6 +625,7 @@ complecs <- function(input,
     }
   }
 
+  res$output$dot_code <- dot_code;
   res$output$graph <- graph;
   res$output$graphSvg <- graphSvg;
 
@@ -592,9 +635,10 @@ complecs <- function(input,
 
   if (returnSvgOnly) {
     return(graphSvg);
-  } else {
+  } else if (returnGraphOnly) {
     return(graph);
-    #return(res);
+  } else {
+    return(res);
   }
 
 }
